import React from 'react';
import { compose, withProps } from "recompose";
import { withScriptjs, withGoogleMap, StreetViewPanorama, GoogleMap, Marker } from "react-google-maps";
import { gameInfo, getGuesses, sendGuess } from "../../services/ggsh.service";
import withGoogleApiKey from "../with-google-key.component";
import RoundTimer from "../round-timer.component";
import Loading from '../loading.component';

// TODO want a button for moving the pano back to start somehow

const getMapUrl = googleApiKey => `https://maps.googleapis.com/maps/api/js?key=${googleApiKey}&v=3.exp&libraries=geometry,drawing,places`

const PositionedStreetView = compose(
  withProps({
    loadingElement: <div style={{ height: `100%` }} />,
    containerElement: <div style={{ height: "100%" }} />,
    mapElement: <div style={{ height: `100%` }} />,
  }),
  withScriptjs,
  withGoogleMap
)(({ point }) =>
  <StreetViewPanorama
    defaultPosition={point}
    options={{
      addressControl: false,
      showRoadLabels: false,
      clickToGo: true,
    }}
    visible
  />
);

const SelectionMap = compose(
  withProps({
    loadingElement: <div style={{ height: `100%` }} />,
    containerElement: <div style={{ height: `100%` }} />,
    mapElement: <div style={{ height: `100%` }} />,
  }),
  withScriptjs,
  withGoogleMap
)(({onClick, markerPosition}) =>
  <GoogleMap
    defaultZoom={1}
    defaultCenter={{ lat: 25, lng: -25}}
    clickableIcons={false}
    options={{
      disableDefaultUI: true,
      fullscreenControl: true,
    }}
    onClick={onClick}>
      {markerPosition ? <Marker
        clickable={false}
        draggable={false}
        position={markerPosition}
      /> : null}
  </GoogleMap>
);

const GamePanel = ({
  googleApiKey,
  onSelectPoint,
  onSubmitGuess,
  streetViewPoint,
  selectedPoint,
  roundSeconds,
  onTimeout,
  submitDisabled,
}) => (
  <div style={{
    height: "100%",
    display: "flex",
    justifyContent: "space-between",
    alignItems: "center"
  }}>
    <div style={{ height: "100%", margin: "2px", flex: 3 }}>
      <div style={{ position: "relative", height: "100%" }}>
        <PositionedStreetView
          googleMapURL={getMapUrl(googleApiKey)}
          point={streetViewPoint}
        />
      </div>
    </div>
    <div style={{
      height: "100%",
      flex: 1,
      display: "flex",
      flexDirection: "column",
      justifyContent:"space-around",
    }}>
      <RoundTimer style={{ flex: 1 }} seconds={roundSeconds} onTimeout={onTimeout} />
      <div style={{ height: "100%", margin: "5px", flex: 6 }}>
        <SelectionMap
          googleMapURL={getMapUrl(googleApiKey)}
          onClick={({ latLng }) => onSelectPoint({ lat: latLng.lat(), lng: latLng.lng() })}
          markerPosition={selectedPoint}
        />
      </div>
      <button
        style={{ margin: "5px", flex: 1 }}
        onClick={onSubmitGuess}
        disabled={submitDisabled || selectedPoint === null}
      >
        Submit Guess
      </button>
    </div>
  </div>
);

const GamePanelWithApiKey = withGoogleApiKey(GamePanel);

class GamePanelContainer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      currentRound: null,
      targetPoint: null,
      selectedPoint: null,
      roundTimer: null,
      submitDisabled: false,
    }
  }

  async componentDidMount() {
    const { gameId, playerName } = this.props;
    const { currentRound } = await getGuesses(gameId, playerName);
    if (currentRound) {
      const { coords, timer } = await gameInfo(gameId);
      const targetPoint = coords[currentRound];
      this.setState({
        currentRound,
        targetPoint,
        roundTimer: timer,
      });
    } else {
      const { onGameEnd } = this.props;
      onGameEnd();
    }
  }

  async handleSubmitGuess() {
    this.setState({ submitDisabled: true })
    const { currentRound, selectedPoint, targetPoint } = this.state;
    const { gameId, playerName, onRoundEnd } = this.props;
    const { score, totalScore } = await sendGuess(gameId, playerName, currentRound, selectedPoint || { timeout: true });
    onRoundEnd({
      roundNum: currentRound,
      selectedPoint,
      targetPoint,
      score,
      totalScore,
    });
  }

  render() {
    const { 
      currentRound, 
      targetPoint, 
      selectedPoint, 
      roundTimer, 
      submitDisabled 
    } = this.state;
    if (!currentRound || !targetPoint || !roundTimer) {
      return <Loading/>
    }
  
    return (
      <GamePanelWithApiKey
        onSelectPoint={selectedPoint => this.setState({ selectedPoint })}
        onSubmitGuess={() => this.handleSubmitGuess()}
        streetViewPoint={targetPoint}
        selectedPoint={selectedPoint}        
        roundSeconds={roundTimer}
        onTimeout={() => this.handleSubmitGuess()}
        submitDisabled={submitDisabled}
      />
    );
  }
}

export default GamePanelContainer;
